package com.sun.org.apache.xalan.internal.xsltc.runtime;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilderFactory;
import jdk.xml.internal.JdkXmlUtils;
import org.w3c.dom.Document;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.File;
import com.sun.org.apache.xalan.internal.xsltc.runtime.output.TransletOutputHandlerFactory;
import com.sun.org.apache.xml.internal.serializer.SerializationHandler;
import com.sun.org.apache.xml.internal.dtm.DTMAxisIterator;
import java.util.Iterator;
import com.sun.org.apache.xalan.internal.xsltc.DOMEnhancedForDTM;
import java.util.HashMap;
import java.text.DecimalFormatSymbols;
import com.sun.org.apache.xalan.internal.xsltc.TransletException;
import com.sun.org.apache.xalan.internal.xsltc.dom.DOMAdapter;
import com.sun.org.apache.xalan.internal.xsltc.DOM;
import org.w3c.dom.DOMImplementation;
import com.sun.org.apache.xalan.internal.xsltc.DOMCache;
import com.sun.org.apache.xalan.internal.xsltc.dom.KeyIndex;
import java.text.DecimalFormat;
import java.util.Map;
import javax.xml.transform.Templates;
import java.util.ArrayList;
import com.sun.org.apache.xalan.internal.xsltc.Translet;

public abstract class AbstractTranslet implements Translet
{
    public String _version;
    public String _method;
    public String _encoding;
    public boolean _omitHeader;
    public String _standalone;
    public boolean _isStandalone;
    public String _doctypePublic;
    public String _doctypeSystem;
    public boolean _indent;
    public String _mediaType;
    public ArrayList<String> _cdata;
    public int _indentamount;
    public static final int FIRST_TRANSLET_VERSION = 100;
    public static final int VER_SPLIT_NAMES_ARRAY = 101;
    public static final int CURRENT_TRANSLET_VERSION = 101;
    protected int transletVersion;
    protected String[] namesArray;
    protected String[] urisArray;
    protected int[] typesArray;
    protected String[] namespaceArray;
    protected Templates _templates;
    protected boolean _hasIdCall;
    protected StringValueHandler stringValueHandler;
    private static final String EMPTYSTRING = "";
    private static final String ID_INDEX_NAME = "##id";
    private boolean _overrideDefaultParser;
    private String _accessExternalStylesheet;
    protected int pbase;
    protected int pframe;
    protected ArrayList paramsStack;
    private MessageHandler _msgHandler;
    public Map<String, DecimalFormat> _formatSymbols;
    private Map<String, KeyIndex> _keyIndexes;
    private KeyIndex _emptyKeyIndex;
    private int _indexSize;
    private int _currentRootForKeys;
    private DOMCache _domCache;
    private Map<String, Class<?>> _auxClasses;
    protected DOMImplementation _domImplementation;
    
    public AbstractTranslet() {
        this._version = "1.0";
        this._method = null;
        this._encoding = "UTF-8";
        this._omitHeader = false;
        this._standalone = null;
        this._isStandalone = false;
        this._doctypePublic = null;
        this._doctypeSystem = null;
        this._indent = false;
        this._mediaType = null;
        this._cdata = null;
        this._indentamount = -1;
        this.transletVersion = 100;
        this._templates = null;
        this._hasIdCall = false;
        this.stringValueHandler = new StringValueHandler();
        this._accessExternalStylesheet = "all";
        this.pbase = 0;
        this.pframe = 0;
        this.paramsStack = new ArrayList();
        this._msgHandler = null;
        this._formatSymbols = null;
        this._keyIndexes = null;
        this._emptyKeyIndex = null;
        this._indexSize = 0;
        this._currentRootForKeys = 0;
        this._domCache = null;
        this._auxClasses = null;
        this._domImplementation = null;
    }
    
    public void printInternalState() {
        System.out.println("-------------------------------------");
        System.out.println("AbstractTranslet this = " + this);
        System.out.println("pbase = " + this.pbase);
        System.out.println("vframe = " + this.pframe);
        System.out.println("paramsStack.size() = " + this.paramsStack.size());
        System.out.println("namesArray.size = " + this.namesArray.length);
        System.out.println("namespaceArray.size = " + this.namespaceArray.length);
        System.out.println("");
        System.out.println("Total memory = " + Runtime.getRuntime().totalMemory());
    }
    
    public final DOMAdapter makeDOMAdapter(DOM dom) throws TransletException {
        this.setRootForKeys(dom.getDocument());
        return new DOMAdapter(dom, this.namesArray, this.urisArray, this.typesArray, this.namespaceArray);
    }
    
    public final void pushParamFrame() {
        this.paramsStack.add(this.pframe, Integer.valueOf(this.pbase));
        int n = this.pframe + 1;
        this.pframe = n;
        this.pbase = n;
    }
    
    public final void popParamFrame() {
        if (this.pbase > 0) {
            ArrayList<Object> paramsStack = this.paramsStack;
            int n = this.pbase - 1;
            this.pbase = n;
            int oldpbase = (int)paramsStack.get(n);
            for (int i = this.pframe - 1; i >= this.pbase; --i) {
                this.paramsStack.remove(i);
            }
            this.pframe = this.pbase;
            this.pbase = oldpbase;
        }
    }
    
    @Override
    public final Object addParameter(String name, Object value) {
        name = BasisLibrary.mapQNameToJavaName(name);
        return this.addParameter(name, value, false);
    }
    
    public final Object addParameter(String name, Object value, boolean isDefault) {
        int i = this.pframe - 1;
        while (i >= this.pbase) {
            Parameter param = (Parameter)this.paramsStack.get(i);
            if (param._name.equals(name)) {
                if (param._isDefault || !isDefault) {
                    param._value = value;
                    param._isDefault = isDefault;
                    return value;
                }
                return param._value;
            }
            else {
                --i;
            }
        }
        this.paramsStack.add(this.pframe++, new Parameter(name, value, isDefault));
        return value;
    }
    
    public void clearParameters() {
        int n = 0;
        this.pframe = n;
        this.pbase = n;
        this.paramsStack.clear();
    }
    
    public final Object getParameter(String name) {
        name = BasisLibrary.mapQNameToJavaName(name);
        int i = this.pframe - 1;
        while (i >= this.pbase) {
            Parameter param = (Parameter)this.paramsStack.get(i);
            if (param._name.equals(name)) {
                return param._value;
            }
            --i;
        }
        return null;
    }
    
    public final void setMessageHandler(MessageHandler handler) {
        this._msgHandler = handler;
    }
    
    public final void displayMessage(String msg) {
        if (this._msgHandler == null) {
            System.err.println(msg);
        }
        else {
            this._msgHandler.displayMessage(msg);
        }
    }
    
    public void addDecimalFormat(String name, DecimalFormatSymbols symbols) {
        if (this._formatSymbols == null) {
            this._formatSymbols = new HashMap();
        }
        if (name == null) {
            name = "";
        }
        DecimalFormat df = new DecimalFormat();
        if (symbols != null) {
            df.setDecimalFormatSymbols(symbols);
        }
        this._formatSymbols.put(name, df);
    }
    
    public final DecimalFormat getDecimalFormat(String name) {
        if (this._formatSymbols != null) {
            if (name == null) {
                name = "";
            }
            DecimalFormat df = (DecimalFormat)this._formatSymbols.get(name);
            if (df == null) {
                df = (DecimalFormat)this._formatSymbols.get("");
                return df;
            }
            return df;
        }
        else {
            return null;
        }
    }
    
    public final void prepassDocument(DOM document) {
        this.setIndexSize(document.getSize());
        this.buildIDIndex(document);
    }
    
    private final void buildIDIndex(DOM document) {
        this.setRootForKeys(document.getDocument());
        if ((document instanceof DOMEnhancedForDTM)) {
            DOMEnhancedForDTM enhancedDOM = (DOMEnhancedForDTM)document;
            if (enhancedDOM.hasDOMSource()) {
                this.buildKeyIndex("##id", document);
                return;
            }
            Map<String, Integer> elementsByID = enhancedDOM.getElementsWithIDs();
            if (elementsByID == null) {
                return;
            }
            boolean hasIDValues = false;
            for (Map.Entry<String, Integer> entry : elementsByID.entrySet()) {
                int element = document.getNodeHandle((int)entry.getValue());
                this.buildKeyIndex("##id", element, (String)entry.getKey());
                hasIDValues = true;
            }
            if (hasIDValues) {
                this.setKeyIndexDom("##id", document);
            }
        }
    }
    
    public final void postInitialization() {
        if (this.transletVersion < 101) {
            int arraySize = this.namesArray.length;
            String[] newURIsArray = new String[arraySize];
            String[] newNamesArray = new String[arraySize];
            int[] newTypesArray = new int[arraySize];
            String name;
            int lNameStartIdx;
            for (int i = 0; i < arraySize; newNamesArray[i] = ((lNameStartIdx == 0) ? name : name.substring(lNameStartIdx)), ++i) {
                name = this.namesArray[i];
                int colonIndex = name.lastIndexOf(58);
                lNameStartIdx = colonIndex + 1;
                if (colonIndex > -1) {
                    newURIsArray[i] = name.substring(0, colonIndex);
                }
                if (name.charAt(lNameStartIdx) == '@') {
                    ++lNameStartIdx;
                    newTypesArray[i] = 2;
                }
                else if (name.charAt(lNameStartIdx) == '?') {
                    ++lNameStartIdx;
                    newTypesArray[i] = 13;
                }
                else {
                    newTypesArray[i] = 1;
                }
            }
            this.namesArray = newNamesArray;
            this.urisArray = newURIsArray;
            this.typesArray = newTypesArray;
        }
        if (this.transletVersion > 101) {
            BasisLibrary.runTimeError("UNKNOWN_TRANSLET_VERSION_ERR", this.getClass().getName());
        }
    }
    
    public void setIndexSize(int size) {
        if (size > this._indexSize) {
            this._indexSize = size;
        }
    }
    
    public KeyIndex createKeyIndex() {
        return new KeyIndex(this._indexSize);
    }
    
    public void buildKeyIndex(String name, int node, String value) {
        KeyIndex index = this.buildKeyIndexHelper(name);
        index.add(value, node, this._currentRootForKeys);
    }
    
    public void buildKeyIndex(String name, DOM dom) {
        KeyIndex index = this.buildKeyIndexHelper(name);
        index.setDom(dom, dom.getDocument());
    }
    
    private KeyIndex buildKeyIndexHelper(String name) {
        if (this._keyIndexes == null) {
            this._keyIndexes = new HashMap();
        }
        KeyIndex index = (KeyIndex)this._keyIndexes.get(name);
        if (index == null) {
            this._keyIndexes.put(name, index = new KeyIndex(this._indexSize));
            return index;
        }
        return index;
    }
    
    public KeyIndex getKeyIndex(String name) {
        if (this._keyIndexes == null) {
            return (this._emptyKeyIndex != null) ? this._emptyKeyIndex : (this._emptyKeyIndex = new KeyIndex(1));
        }
        KeyIndex index = (KeyIndex)this._keyIndexes.get(name);
        if (index == null) {
            return (this._emptyKeyIndex != null) ? this._emptyKeyIndex : (this._emptyKeyIndex = new KeyIndex(1));
        }
        return index;
    }
    
    private void setRootForKeys(int root) {
        this._currentRootForKeys = root;
    }
    
    @Override
    public void buildKeys(DOM document, DTMAxisIterator iterator, SerializationHandler handler, int root) throws TransletException {
    }
    
    public void setKeyIndexDom(String name, DOM document) {
        this.getKeyIndex(name).setDom(document, document.getDocument());
    }
    
    public void setDOMCache(DOMCache cache) {
        this._domCache = cache;
    }
    
    public DOMCache getDOMCache() {
        return this._domCache;
    }
    
    public SerializationHandler openOutputHandler(String filename, boolean append) throws TransletException {
        try {
            TransletOutputHandlerFactory factory = TransletOutputHandlerFactory.newInstance(this._overrideDefaultParser);
            String dirStr = new File(filename).getParent();
            if (null != dirStr && dirStr.length() > 0) {
                File dir = new File(dirStr);
                dir.mkdirs();
            }
            factory.setEncoding(this._encoding);
            factory.setOutputMethod(this._method);
            factory.setOutputStream(new BufferedOutputStream(new FileOutputStream(filename, append)));
            factory.setOutputType(0);
            SerializationHandler handler = factory.getSerializationHandler();
            this.transferOutputSettings(handler);
            handler.startDocument();
            return handler;
        }
        catch (Exception e) {
            throw new TransletException(e);
        }
    }
    
    public SerializationHandler openOutputHandler(String filename) throws TransletException {
        return this.openOutputHandler(filename, false);
    }
    
    public void closeOutputHandler(SerializationHandler handler) {
        try {
            handler.endDocument();
            handler.close();
        }
        catch (Exception ex) {}
    }
    
    @Override
    public abstract void transform(DOM p0, DTMAxisIterator p1, SerializationHandler p2) throws TransletException;
    
    @Override
    public final void transform(DOM document, SerializationHandler handler) throws TransletException {
        try {
            this.transform(document, document.getIterator(), handler);
        }
        finally {
            this._keyIndexes = null;
        }
    }
    
    public final void characters(String string, SerializationHandler handler) throws TransletException {
        if (string != null) {
            try {
                handler.characters(string);
            }
            catch (Exception e) {
                throw new TransletException(e);
            }
        }
    }
    
    public void addCdataElement(String name) {
        if (this._cdata == null) {
            this._cdata = new ArrayList();
        }
        int lastColon = name.lastIndexOf(58);
        if (lastColon > 0) {
            String uri = name.substring(0, lastColon);
            String localName = name.substring(lastColon + 1);
            this._cdata.add(uri);
            this._cdata.add(localName);
        }
        else {
            this._cdata.add(null);
            this._cdata.add(name);
        }
    }
    
    protected void transferOutputSettings(SerializationHandler handler) {
        if (this._method != null) {
            if (this._method.equals("xml")) {
                if (this._standalone != null) {
                    handler.setStandalone(this._standalone);
                }
                if (this._omitHeader) {
                    handler.setOmitXMLDeclaration(true);
                }
                handler.setCdataSectionElements(this._cdata);
                if (this._version != null) {
                    handler.setVersion(this._version);
                }
                handler.setIndent(this._indent);
                handler.setIndentAmount(this._indentamount);
                if (this._doctypeSystem != null) {
                    handler.setDoctype(this._doctypeSystem, this._doctypePublic);
                }
                handler.setIsStandalone(this._isStandalone);
            }
            else if (this._method.equals("html")) {
                handler.setIndent(this._indent);
                handler.setDoctype(this._doctypeSystem, this._doctypePublic);
                if (this._mediaType != null) {
                    handler.setMediaType(this._mediaType);
                }
            }
        }
        else {
            handler.setCdataSectionElements(this._cdata);
            if (this._version != null) {
                handler.setVersion(this._version);
            }
            if (this._standalone != null) {
                handler.setStandalone(this._standalone);
            }
            if (this._omitHeader) {
                handler.setOmitXMLDeclaration(true);
            }
            handler.setIndent(this._indent);
            handler.setDoctype(this._doctypeSystem, this._doctypePublic);
            handler.setIsStandalone(this._isStandalone);
        }
    }
    
    @Override
    public void addAuxiliaryClass(Class auxClass) {
        if (this._auxClasses == null) {
            this._auxClasses = new HashMap();
        }
        this._auxClasses.put(auxClass.getName(), auxClass);
    }
    
    public void setAuxiliaryClasses(Map<String, Class<?>> auxClasses) {
        this._auxClasses = auxClasses;
    }
    
    @Override
    public Class getAuxiliaryClass(String className) {
        if (this._auxClasses == null) {
            return null;
        }
        return (Class)this._auxClasses.get(className);
    }
    
    @Override
    public String[] getNamesArray() {
        return this.namesArray;
    }
    
    @Override
    public String[] getUrisArray() {
        return this.urisArray;
    }
    
    @Override
    public int[] getTypesArray() {
        return this.typesArray;
    }
    
    @Override
    public String[] getNamespaceArray() {
        return this.namespaceArray;
    }
    
    public boolean hasIdCall() {
        return this._hasIdCall;
    }
    
    public Templates getTemplates() {
        return this._templates;
    }
    
    public void setTemplates(Templates templates) {
        this._templates = templates;
    }
    
    @Override
    public boolean overrideDefaultParser() {
        return this._overrideDefaultParser;
    }
    
    @Override
    public void setOverrideDefaultParser(boolean flag) {
        this._overrideDefaultParser = flag;
    }
    
    public String getAllowedProtocols() {
        return this._accessExternalStylesheet;
    }
    
    public void setAllowedProtocols(String protocols) {
        this._accessExternalStylesheet = protocols;
    }
    
    public Document newDocument(String uri, String qname) throws ParserConfigurationException {
        if (this._domImplementation == null) {
            DocumentBuilderFactory dbf = JdkXmlUtils.getDOMFactory(this._overrideDefaultParser);
            this._domImplementation = dbf.newDocumentBuilder().getDOMImplementation();
        }
        return this._domImplementation.createDocument(uri, qname, null);
    }
}
